from pygments.lexer import RegexLexer, bygroups, include
from pygments.token import *

class AperiotLexer(RegexLexer):
    name = 'aperiot'
    aliases = ['aperiot', 'apr']
    filenames = ['*.apr']
    #mimetypes = ['text/x-klt', 'application/x-klt']

    tokens = {
        'root': [
            (r'\n', Text),
            (r'[^\S\n]+', Text),
            (r'#.*\n', Comment),
            (r':|->', Punctuation),
            (r'(indent|dedent|empty|newline)', Operator.Word),
            (r'(import|useindents|usenewlines|'
             r'keywords|operators|separators|brackets|identifiers|numbers|strings|'
             r'start|rules)', Keyword),
            ('"', String, 'dqs')
        ],
        'strings': [
            (r'%(\([a-zA-Z0-9]+\))?[-#0 +]*([0-9]+|[*])?(\.([0-9]+|[*]))?'
             '[hlL]?[diouxXeEfFgGcrs%]', String.Interpol),
            (r'[^\\\'"%\n]+', String),
            # quotes, percents and backslashes must be parsed one at a time
            (r'[\'"\\]', String),
            # unhandled string formatting sign
            (r'%', String)
            # newlines are an error (use "nl" state)
        ],
        'nl': [
            (r'\n', String)
        ],
        'dqs': [
            (r'"', String, '#pop'),
            (r'\\\\|\\"|\\\n', String.Escape), # included here again for raw strings
            include('strings')
        ],
    }

from pygments.style import Style

class AperiotStyle(Style):
    default_style = ""
    styles = {
        Text:                   '#000000',
        Comment:                'italic #adadad',
        Keyword:                'bold #020280',
        Name:                   '#000000',
        Name.Function:          '#000000',
        Name.Class:             '#287234',
        String:                 '#e5aeae',
        String.Interpol:        '#e5aeae',
        Punctuation:            '#287234',
        Operator:               '#287234',
        Operator.Word:          '#287234',
        Number:                 '#000',
        Number.Float:           '#000',
        Number.Integer:         '#000',
    }

import sys

from pygments import highlight
from pygments.formatters import LatexFormatter, HtmlFormatter

import getopt

def main():
    try:
        options, args = getopt.getopt(sys.argv[1:], 'fhp:r',[])
        full = False
        html = False
        preamble = None
        renew = False
        for option, arg in options:
            if option == '-f':
                full = True
            elif option == '-h':
                html = True
            elif option == '-p':
                preamble = arg
            elif option == '-r':
                renew = True
        
        lexer = AperiotLexer()
        if html:
            formatter = HtmlFormatter(full=full,style=AperiotStyle)
            extension = '.html'
        else:
            formatter = LatexFormatter(full=full,style=AperiotStyle)
            extension = '.tex'
        
        if preamble is None:
            filename = args[0]
            f = open(filename,'r')
            code = f.read()
            f.close()
            import os.path
            root,ext = os.path.splitext(filename)
            new_filename = root + extension
            nf = open(new_filename,'w')
            text = highlight(code, lexer, formatter)
            nf.write(text)
            nf.close()
        else:
            arg = ''
            if renew:
                arg = True
            text = formatter.get_style_defs(arg=arg)
            f = open(preamble, 'w')
            f.write(text)
            f.close()

    except getopt.GetoptError, e:
        print "Bad options:",e
    except IndexError:
        print 'No file given.'
    except IOError, e:
        print e



main()
