
from aperiot.lexer import *

imports = ['aperiot.lexer', 'quotefix']

useindents = True
usenewlines = True

# Terminal symbols

t_indent  = Indent
t_dedent  = Dedent
t_newline = Newline
t_importkwd      = Keyword('import', 'importkwd')
t_lexerkwd       = Keyword('uselexer', 'lexerkwd')
t_useindentskwd  = Keyword('useindents', 'useindentskwd')
t_usenewlineskwd = Keyword('usenewlines', 'usenewlineskwd')
t_keywordskwd    = Keyword('keywords', 'keywordskwd')
t_operatorskwd   = Keyword('operators', 'operatorskwd')
t_separatorskwd  = Keyword('separators', 'separatorskwd')
t_bracketskwd    = Keyword('brackets', 'bracketskwd')
t_identifierskwd = Keyword('identifiers', 'identifierskwd')
t_numberskwd     = Keyword('numbers', 'numberskwd')
t_stringskwd     = Keyword('strings', 'stringskwd')
t_startkwd       = Keyword('start', 'startkwd')
t_ruleskwd       = Keyword('rules', 'ruleskwd')
t_emptykwd       = Keyword('empty', 'emptykwd')
t_arrow = Operator('->', 'arrow')
t_colon = Separator(':', 'colon')
t_bar   = Separator('|', 'bar')
t_dot   = Separator('.', 'dot')
t_nonterminal = Identifier
t_name        = Identifier
t_terminal = StringLiteral
t_action   = StringLiteral


# Non-terminal symbols

n_KEYWORD              = 'KEYWORD'
n_KEYWORDS_SECTION     = 'KEYWORDS_SECTION'
n_SYMBOL               = 'SYMBOL'
n_NUMBER               = 'NUMBER'
n_BRACKETS_SECTION     = 'BRACKETS_SECTION'
n_NUMBERS_SECTION      = 'NUMBERS_SECTION'
n_LIST_OF_ALTERNATIVES = 'LIST_OF_ALTERNATIVES'
n_IDENTIFIER_LINE      = 'IDENTIFIER_LINE'
n_OPERATORS_SECTION    = 'OPERATORS_SECTION'
n_STRING               = 'STRING'
n_SEPARATORS_LINES     = 'SEPARATORS_LINES'
n_TERMINAL             = 'TERMINAL'
n_WORD                 = 'WORD'
n_OPERATORS_LINES      = 'OPERATORS_LINES'
n_STRINGS_SECTION      = 'STRINGS_SECTION'
n_KEYWORD_LINE         = 'KEYWORD_LINE'
n_RULES_SECTION        = 'RULES_SECTION'
n_KEYWORD_LINES        = 'KEYWORD_LINES'
n_IMPORT               = 'IMPORT'
n_SEPARATORS_SECTION   = 'SEPARATORS_SECTION'
n_IDENTIFIER           = 'IDENTIFIER'
n_RULE_LIST            = 'RULE_LIST'
n_BRACKETS_LINES       = 'BRACKETS_LINES'
n_RULE                 = 'RULE'
n_GRAMMAR              = 'GRAMMAR'
n_LEXER_MODULE         = 'LEXER_MODULE'
n_STRINGS_LINES        = 'STRINGS_LINES'
n_USEINDENTS           = 'USEINDENTS'
n_LEXER_SECTION        = 'LEXER_SECTION'
n_TERMINALS_SECTIONS   = 'TERMINALS_SECTIONS'
n_START_SECTION        = 'START_SECTION'
n_TERMINALS_SECTION    = 'TERMINALS_SECTION'
n_BRACKETS_LINE        = 'BRACKETS_LINE'
n_RULE_TABLE           = 'RULE_TABLE'
n_OPERATORS_LINE       = 'OPERATORS_LINE'
n_IDENTIFIERS_SECTION  = 'IDENTIFIERS_SECTION'
n_USENEWLINES          = 'USENEWLINES'
n_ALL                  = 'ALL'
n_IMPORTS_LINES        = 'IMPORTS_LINES'
n_SYMBOL_LIST          = 'SYMBOL_LIST'
n_RHS                  = 'RHS'
n_SEPARATORS_LINE      = 'SEPARATORS_LINE'
n_NUMBERS_LINES        = 'NUMBERS_LINES'
n_EMPTY_LINES          = 'EMPTY_LINES'
n_IMPORTS_SECTION      = 'IMPORTS_SECTION'
n_IMPORTS_LINE         = 'IMPORTS_LINE'
n_IDENTIFIER_LINES     = 'IDENTIFIER_LINES'
n_NUMBERS_LINE         = 'NUMBERS_LINE'
n_ALTERNATIVE          = 'ALTERNATIVE'
n_NONTERMINAL          = 'NONTERMINAL'
n_STRINGS_LINE         = 'STRINGS_LINE'


# Start symbol

start = 'ALL'

# Rules

rules = \
    {
        n_KEYWORD: 
            [
                ([t_terminal], 'Keyword($1.string)'), 
                ([t_name, t_terminal], 'Keyword($2.string, $1.string)'), 
            ],
        n_KEYWORDS_SECTION: 
            [
                ([t_keywordskwd, t_newline, t_indent, n_KEYWORD_LINES, t_dedent], '$4'), 
            ],
        n_SYMBOL: 
            [
                ([n_TERMINAL], '$1'), 
                ([n_NONTERMINAL], '$1'), 
            ],
        n_NUMBER: 
            [
                ([t_name], 'Token($1.string)'), 
            ],
        n_BRACKETS_SECTION: 
            [
                ([t_bracketskwd, t_newline, t_indent, n_BRACKETS_LINES, t_dedent], '$4'), 
            ],
        n_NUMBERS_SECTION: 
            [
                ([t_numberskwd, t_newline, t_indent, n_NUMBERS_LINES, t_dedent], '$4'), 
            ],
        n_LIST_OF_ALTERNATIVES: 
            [
                ([t_bar, n_ALTERNATIVE, t_newline], '[$2]'), 
                ([t_bar, n_ALTERNATIVE, t_newline, n_LIST_OF_ALTERNATIVES], '[$2] + $4'), 
            ],
        n_IDENTIFIER_LINE: 
            [
                ([n_IDENTIFIER, t_newline], '[$1]'), 
                ([n_IDENTIFIER, n_IDENTIFIER_LINE], '[$1] + $2'), 
            ],
        n_OPERATORS_SECTION: 
            [
                ([t_operatorskwd, t_newline, t_indent, n_OPERATORS_LINES, t_dedent], '$4'), 
            ],
        n_STRING: 
            [
                ([t_name], 'Token($1.string)'), 
            ],
        n_SEPARATORS_LINES: 
            [
                ([n_SEPARATORS_LINE], '[$1]'), 
                ([n_SEPARATORS_LINE, n_SEPARATORS_LINES], '[$1] + $2'), 
            ],
        n_TERMINAL: 
            [
                ([t_terminal], 'Token($1.string)'), 
            ],
        n_WORD: 
            [
                ([n_SYMBOL_LIST], '$1'), 
                ([t_emptykwd], '[]'), 
            ],
        n_OPERATORS_LINES: 
            [
                ([n_OPERATORS_LINE], '[$1]'), 
                ([n_OPERATORS_LINE, n_OPERATORS_LINES], '[$1] + $2'), 
            ],
        n_STRINGS_SECTION: 
            [
                ([t_stringskwd, t_newline, t_indent, n_STRINGS_LINES, t_dedent], '$4'), 
            ],
        n_KEYWORD_LINE: 
            [
                ([n_KEYWORD, t_newline], '[$1]'), 
                ([n_KEYWORD, n_KEYWORD_LINE], '[$1] + $2'), 
            ],
        n_RULES_SECTION: 
            [
                ([t_ruleskwd, n_EMPTY_LINES, n_RULE_TABLE], '$3'), 
            ],
        n_KEYWORD_LINES: 
            [
                ([n_KEYWORD_LINE], '$1'), 
                ([n_KEYWORD_LINE, n_KEYWORD_LINES], '$1 + $2'), 
            ],
        n_IMPORT: 
            [
                ([t_name], '$1.string'), 
                ([t_name, t_dot, n_IMPORT], '$1.string + \'.\' + $3'), 
            ],
        n_SEPARATORS_SECTION: 
            [
                ([t_separatorskwd, t_newline, t_indent, n_SEPARATORS_LINES, t_dedent], '$4'), 
            ],
        n_IDENTIFIER: 
            [
                ([t_name], 'Identifier($1.string)'), 
            ],
        n_RULE_LIST: 
            [
                ([n_RULE], '[$1]'), 
                ([n_RULE, n_RULE_LIST], '[$1] + $2'), 
            ],
        n_BRACKETS_LINES: 
            [
                ([n_BRACKETS_LINE], '[$1]'), 
                ([n_BRACKETS_LINE, n_BRACKETS_LINES], '[$1] + $2'), 
            ],
        n_RULE: 
            [
                ([n_NONTERMINAL, t_arrow, n_RHS], '($1, $3)'), 
            ],
        n_GRAMMAR: 
            [
                ([n_IMPORTS_SECTION, n_LEXER_SECTION, n_USEINDENTS, n_USENEWLINES, n_TERMINALS_SECTIONS, n_START_SECTION, n_RULES_SECTION], '($1, $2, $3, $4, dict($5), $6, $7)'), 
            ],
        n_LEXER_MODULE: 
            [
                ([t_name, t_newline], '$1.string'), 
                ([t_name, t_dot, n_LEXER_MODULE], '$1.string + \'.\' + $3'), 
            ],
        n_STRINGS_LINES: 
            [
                ([n_STRINGS_LINE], '$1'), 
                ([n_STRINGS_LINE, n_STRINGS_LINES], '$1 + $2'), 
            ],
        n_USEINDENTS: 
            [
                ([], 'False'), 
                ([t_useindentskwd, t_newline], 'True'), 
            ],
        n_LEXER_SECTION: 
            [
                ([], 'None'), 
                ([t_lexerkwd, t_newline, t_indent, n_LEXER_MODULE, t_dedent], 'fixquotes($4)'), 
            ],
        n_TERMINALS_SECTIONS: 
            [
                ([], '[]'), 
                ([n_TERMINALS_SECTION, n_TERMINALS_SECTIONS], '[$1] + $2'), 
            ],
        n_START_SECTION: 
            [
                ([t_startkwd, t_newline, t_indent, n_NONTERMINAL, t_newline, t_dedent], '$4'), 
            ],
        n_TERMINALS_SECTION: 
            [
                ([n_KEYWORDS_SECTION], '(\'keywords\', $1)'), 
                ([n_OPERATORS_SECTION], '(\'operators\', $1)'), 
                ([n_SEPARATORS_SECTION], '(\'separators\', $1)'), 
                ([n_BRACKETS_SECTION], '(\'brackets\', $1)'), 
                ([n_IDENTIFIERS_SECTION], '(\'identifiers\', $1)'), 
                ([n_NUMBERS_SECTION], '(\'numbers\', $1)'), 
                ([n_STRINGS_SECTION], '(\'strings\', $1)'), 
            ],
        n_BRACKETS_LINE: 
            [
                ([t_name, t_terminal, t_newline], 'Bracket($2.string, $1.string)'), 
            ],
        n_RULE_TABLE: 
            [
                ([n_RULE_LIST], 'dict($1)'), 
            ],
        n_OPERATORS_LINE: 
            [
                ([t_name, t_terminal, t_newline], 'Operator($2.string, $1.string)'), 
            ],
        n_IDENTIFIERS_SECTION: 
            [
                ([t_identifierskwd, t_newline, t_indent, n_IDENTIFIER_LINES, t_dedent], '$4'), 
            ],
        n_USENEWLINES: 
            [
                ([], 'False'), 
                ([t_usenewlineskwd, t_newline], 'True'), 
            ],
        n_ALL: 
            [
                ([n_GRAMMAR], '$1'), 
            ],
        n_IMPORTS_LINES: 
            [
                ([n_IMPORTS_LINE], '$1'), 
                ([n_IMPORTS_LINE, n_IMPORTS_LINES], '$1 + $2'), 
            ],
        n_SYMBOL_LIST: 
            [
                ([n_SYMBOL, n_SYMBOL_LIST], '[$1] + $2'), 
                ([n_SYMBOL], '[$1]'), 
            ],
        n_RHS: 
            [
                ([n_ALTERNATIVE, t_newline], '[$1]'), 
                ([n_ALTERNATIVE, t_newline, t_indent, n_LIST_OF_ALTERNATIVES, t_dedent], '[$1] + $4'), 
            ],
        n_SEPARATORS_LINE: 
            [
                ([t_name, t_terminal, t_newline], 'Separator($2.string, $1.string)'), 
            ],
        n_NUMBERS_LINES: 
            [
                ([n_NUMBERS_LINE], '$1'), 
                ([n_NUMBERS_LINE, n_NUMBERS_LINES], '$1 + $2'), 
            ],
        n_EMPTY_LINES: 
            [
                ([t_newline], 'None'), 
                ([t_newline, n_EMPTY_LINES], 'None'), 
            ],
        n_IMPORTS_SECTION: 
            [
                ([], '[]'), 
                ([t_importkwd, t_newline, t_indent, n_IMPORTS_LINES, t_dedent], '$4'), 
            ],
        n_IMPORTS_LINE: 
            [
                ([n_IMPORT, t_newline], '[$1]'), 
                ([n_IMPORT, n_IMPORTS_LINE], '[$1] + $2'), 
            ],
        n_IDENTIFIER_LINES: 
            [
                ([n_IDENTIFIER_LINE], '$1'), 
                ([n_IDENTIFIER_LINE, n_IDENTIFIER_LINES], '$1 + $2'), 
            ],
        n_NUMBERS_LINE: 
            [
                ([n_NUMBER, t_newline], '[$1]'), 
                ([n_NUMBER, n_NUMBERS_LINE], '[$1] + $2'), 
            ],
        n_ALTERNATIVE: 
            [
                ([n_WORD], '($1, None)'), 
                ([n_WORD, t_colon, t_action], '($1, fixquotes($3.value))'), 
            ],
        n_NONTERMINAL: 
            [
                ([t_nonterminal], '$1.string'), 
            ],
        n_STRINGS_LINE: 
            [
                ([n_STRING, t_newline], '[$1]'), 
                ([n_STRING, n_STRINGS_LINE], '[$1] + $2'), 
            ],
    }

